@@ -7,7 +7,8 @@ import java.lang.reflect.*;
 import java.net.*;
 import java.security.*;
 import java.util.*;
-import java.util.Map.*;
+import java.util.Map.Entry;
+import java.util.regex.*;
 
 import org.osgi.framework.*;
 import org.osgi.framework.launch.*;
@@ -22,72 +23,121 @@ import aQute.launcher.minifw.*;
  * 1.4.
  */
 public class Launcher implements ServiceListener {
-	private PrintStream						out;
-	private LauncherConstants				parms;
-	private Framework						systemBundle;
-	private final Properties				properties;
-	private boolean							security;
-	private SimplePermissionPolicy			policy;
-	private Runnable						mainThread;
-	private PackageAdmin					padmin;
-	private static File						propertiesFile;
-	private final Timer						timer				= new Timer();
-	private final List<BundleActivator>		embedded			= new ArrayList<BundleActivator>();
-	private TimerTask						watchdog			= null;
-	private final Map<Bundle, Throwable>	errors				= new HashMap<Bundle, Throwable>();
-	private final Map<File, Bundle>			installedBundles	= new LinkedHashMap<File, Bundle>();
+	
+	// Use our own constant for this rather than depend on OSGi core 4.3
+	private static final String FRAMEWORK_SYSTEM_CAPABILITIES_EXTRA = "org.osgi.framework.system.capabilities.extra";
+	
+	private PrintStream					out;
+	LauncherConstants					parms;
+	Framework							systemBundle;
+	volatile boolean					inrefresh;
+	private final Properties			properties;
+	private boolean						security;
+	private SimplePermissionPolicy		policy;
+	private Runnable					mainThread;
+	private PackageAdmin				padmin;
+	private final List<BundleActivator>	embedded			= new ArrayList<BundleActivator>();
+	private final Map<Bundle,Throwable>	errors				= new HashMap<Bundle,Throwable>();
+	private final Map<File,Bundle>		installedBundles	= new LinkedHashMap<File,Bundle>();
+	private File						home				= new File(System.getProperty("user.home"));
+	private File						bnd					= new File(home, "bnd");
+	private List<Bundle>				wantsToBeStarted	= new ArrayList<Bundle>();
 
 	public static void main(String[] args) {
 		try {
+			final InputStream in;
+			final File propertiesFile;
+
 			String path = System.getProperty(LauncherConstants.LAUNCHER_PROPERTIES);
-			assert path != null;
-			propertiesFile = new File(path).getAbsoluteFile();
-			propertiesFile.deleteOnExit();
-			FileInputStream in = new FileInputStream(propertiesFile);
+			if (path != null) {
+				Matcher matcher = Pattern.compile("^([\"'])(.*)\\1$").matcher(path);
+				if (matcher.matches()) {
+					path = matcher.group(2);
+				}
+
+				propertiesFile = new File(path).getAbsoluteFile();
+				if (!propertiesFile.isFile())
+					errorAndExit("Specified launch file `%s' was not found - absolutePath='%s'", path, propertiesFile.getAbsolutePath());
+				in = new FileInputStream(propertiesFile);
+			} else {
+				propertiesFile = null;
+				in = Launcher.class.getClassLoader().getResourceAsStream(DEFAULT_LAUNCHER_PROPERTIES);
+				if (in == null) {
+					printUsage();
+					errorAndExit("Launch file not specified, and no embedded properties found.");
+				}
+			}
+
 			Properties properties = new Properties();
 			try {
 				properties.load(in);
-			} finally {
-				in.close();
 			}
-			Launcher target = new Launcher(properties);
+			finally {
+				if (in != null)
+					in.close();
+			}
+			Launcher target = new Launcher(properties, propertiesFile);
 			target.run(args);
-		} catch (Throwable t) {
+		}
+		catch (Throwable t) {
 			// Last resort ... errors should be handled lower
 			t.printStackTrace(System.err);
 		}
 	}
 
-	public Launcher(Properties properties) throws Exception {
+	private static void printUsage() {
+		System.out.println("Usage: java -Dlauncher.properties=<launcher.properties> -jar <launcher.jar>");
+	}
+
+	private static void errorAndExit(String message, Object... args) {
+		System.err.println(String.format(message, args));
+		System.exit(1);
+	}
+
+	public Launcher(Properties properties, final File propertiesFile) throws Exception {
 		this.properties = properties;
+
+		// Allow the system to override any properties with -Dkey=value
+
+		for (Object key : properties.keySet()) {
+			String s = (String) key;
+			String v = System.getProperty(s);
+			if (v != null)
+				properties.put(key, v);
+		}
+
 		System.getProperties().putAll(properties);
 		this.parms = new LauncherConstants(properties);
 
 		out = System.err;
 		trace("inited runbundles=%s activators=%s timeout=%s", parms.runbundles, parms.activators, parms.timeout);
-		watchdog = new TimerTask() {
-			long	begin	= propertiesFile.lastModified();
+		if (propertiesFile != null && parms.embedded == false) {
+			TimerTask watchdog = new TimerTask() {
+				long	begin	= propertiesFile.lastModified();
 
-			public void run() {
-				if (begin < propertiesFile.lastModified()) {
-					try {
-						FileInputStream in = new FileInputStream(propertiesFile);
-						Properties properties = new Properties();
+				public void run() {
+					if (begin < propertiesFile.lastModified()) {
 						try {
-							properties.load(in);
-						} finally {
-							in.close();
+							FileInputStream in = new FileInputStream(propertiesFile);
+							Properties properties = new Properties();
+							try {
+								properties.load(in);
+							}
+							finally {
+								in.close();
+							}
+							parms = new LauncherConstants(properties);
+							update();
 						}
-						parms = new LauncherConstants(properties);						
-						update();
-					} catch (Exception e) {
-						error("Error in updating the framework from the properties: %s", e);
+						catch (Exception e) {
+							error("Error in updating the framework from the properties: %s", e);
+						}
+						begin = propertiesFile.lastModified();
 					}
-					begin = propertiesFile.lastModified();
 				}
-			}
-		};
-		timer.scheduleAtFixedRate(watchdog, 5000, 1000);
+			};
+			new Timer().scheduleAtFixedRate(watchdog, 5000, 1000);
+		}
 	}
 
 	private void run(String args[]) throws Throwable {
@@ -103,11 +153,13 @@ public class Launcher implements ServiceListener {
 			// Register the command line with ourselves as the
 			// service.
 			if (parms.services) { // Does not work for our dummy framework
-				Hashtable<String, Object> argprops = new Hashtable<String, Object>();
+				Hashtable<String,Object> argprops = new Hashtable<String,Object>();
 				argprops.put(LauncherConstants.LAUNCHER_ARGUMENTS, args);
 				argprops.put(LauncherConstants.LAUNCHER_READY, "true");
-				systemBundle.getBundleContext().registerService(Launcher.class.getName(), this,
-						argprops);
+				argprops.put(Constants.SERVICE_RANKING, -1000);
+				systemBundle.getBundleContext().registerService(new String[] {
+						Object.class.getName(), Launcher.class.getName()
+				}, this, argprops);
 				trace("registered launcher with arguments for syncing");
 			}
 
@@ -122,10 +174,12 @@ public class Launcher implements ServiceListener {
 			}
 			trace("Will run %s as main thread", mainThread);
 			mainThread.run();
-		} catch (Throwable e) {
+		}
+		catch (Throwable e) {
 			error("Unexpected error in the run body: %s", e);
 			throw e;
-		} finally {
+		}
+		finally {
 			systemBundle.stop();
 			trace("stopped system bundle due to leaving run body");
 		}
@@ -165,14 +219,7 @@ public class Launcher implements ServiceListener {
 		} else
 			trace("could not get package admin");
 
-		systemContext.addServiceListener(this,
-				"(&(objectclass=java.lang.Runnable)(main.thread=true))");
-
-		update();
-
-		if (parms.trace) {
-			report(out);
-		}
+		systemContext.addServiceListener(this, "(&(objectclass=java.lang.Runnable)(main.thread=true))");
 
 		// Start embedded activators
 		trace("start embedded activators");
@@ -180,22 +227,30 @@ public class Launcher implements ServiceListener {
 			ClassLoader loader = getClass().getClassLoader();
 			for (Object token : parms.activators) {
 				try {
-					Class<?> clazz = loader.loadClass((String) token);
+					Class< ? > clazz = loader.loadClass((String) token);
 					BundleActivator activator = (BundleActivator) clazz.newInstance();
 					embedded.add(activator);
 					trace("adding activator %s", activator);
-				} catch (Exception e) {
-					throw new IllegalArgumentException("Embedded Bundle Activator incorrect: "
-							+ token + ", " + e);
+				}
+				catch (Exception e) {
+					throw new IllegalArgumentException("Embedded Bundle Activator incorrect: " + token + ", " + e);
 				}
 			}
 		}
+
+		update();
+
+		if (parms.trace) {
+			report(out);
+		}
+
 		int result = LauncherConstants.OK;
 		for (BundleActivator activator : embedded)
 			try {
 				trace("starting activator %s", activator);
 				activator.start(systemContext);
-			} catch (Exception e) {
+			}
+			catch (Exception e) {
 				error("Starting activator %s : %s", activator, e);
 				result = LauncherConstants.ERROR;
 			}
@@ -204,14 +259,72 @@ public class Launcher implements ServiceListener {
 	}
 
 	/**
-	 * @param systemContext
-	 * @throws MalformedURLException
-	 * @throws FileNotFoundException
-	 * @throws IOException
+	 * Ensure that all the bundles in the parameters are actually started. We
+	 * can start in embedded mode (bundles are inside our main jar) or in file
+	 * system mode.
 	 */
-	private void update() throws Exception {
+	void update() throws Exception {
 		trace("Updating framework with %s", parms.runbundles);
+
+		List<Bundle> tobestarted = new ArrayList<Bundle>();
+		if (parms.embedded)
+			installEmbedded(tobestarted);
+		else
+			synchronizeFiles(tobestarted);
+
+		if (padmin != null) {
+			inrefresh = true;
+			padmin.refreshPackages(null);
+			trace("Waiting for refresh to finish");
+
+			// Will be reset by the Framework listener we added
+			// when we created the framework.
+			while (inrefresh)
+				Thread.sleep(100);
+		} else
+			trace("cannot refresh the bundles because there is no Package Admin");
+
+		trace("bundles administered %s", installedBundles.keySet());
+
+		// From now on, the bundles are on their own. They have
+		// by default AllPermission, but if they install bundles
+		// they will not automatically get AllPermission anymore
+		if (security)
+			policy.setDefaultPermissions(null);
+
+		// Get the resolved status
+		if (padmin != null && padmin.resolveBundles(null) == false) {
+			error("could not resolve the bundles");
+			// return LauncherConstants.RESOLVE_ERROR;
+		}
+
+		// Now start all the installed bundles in the same order
+		// (unless they're a fragment)
+
+		trace("Will start bundles: %s", tobestarted);
+		List<Bundle> all = new ArrayList<Bundle>(tobestarted);
+		// Add all bundles that we've tried to start but failed
+		all.addAll(wantsToBeStarted);
 		
+		for (Bundle b : tobestarted) {
+			try {
+				trace("starting %s", b.getSymbolicName());
+				if (!isFragment(b))
+					b.start(Bundle.START_ACTIVATION_POLICY);
+				trace("started  %s", b.getSymbolicName());
+			}
+			catch (BundleException e) {
+				wantsToBeStarted.add(b);
+				error("Failed to start bundle %s-%s, exception %s", b.getSymbolicName(), b.getVersion(), e);
+			}
+		}
+
+	}
+
+	/**
+	 * @param tobestarted
+	 */
+	void synchronizeFiles(List<Bundle> tobestarted) {
 		// Turn the bundle location paths into files
 		List<File> desired = new ArrayList<File>();
 		for (Object o : parms.runbundles) {
@@ -234,14 +347,13 @@ public class Launcher implements ServiceListener {
 		List<File> tobeinstalled = new ArrayList<File>(desired);
 		tobeinstalled.removeAll(installedBundles.keySet());
 
-		List<Bundle> tobestarted = new ArrayList<Bundle>();
-
 		for (File f : tobedeleted)
 			try {
 				trace("uninstalling %s", f);
 				installedBundles.get(f).uninstall();
 				installedBundles.remove(f);
-			} catch (Exception e) {
+			}
+			catch (Exception e) {
 				error("Failed to uninstall bundle %s, exception %s", f, e);
 			}
 
@@ -251,7 +363,8 @@ public class Launcher implements ServiceListener {
 				Bundle b = install(f);
 				installedBundles.put(f, b);
 				tobestarted.add(b);
-			} catch (Exception e) {
+			}
+			catch (Exception e) {
 				error("Failed to uninstall bundle %s, exception %s", f, e);
 			}
 
@@ -260,51 +373,41 @@ public class Launcher implements ServiceListener {
 				Bundle b = installedBundles.get(f);
 				if (b.getLastModified() < f.lastModified()) {
 					trace("updating %s", f);
-					if ( b.getState() == Bundle.ACTIVE) {
+					if (b.getState() == Bundle.ACTIVE) {
 						tobestarted.add(b);
 						b.stop();
 					}
 					b.update();
 				} else
 					trace("bundle is still current according to timestamp %s", f);
-			} catch (Exception e) {
+			}
+			catch (Exception e) {
 				error("Failed to update bundle %s, exception %s", f, e);
 			}
+	}
 
-		if (padmin != null)
-			padmin.refreshPackages(null);
-
-		trace("bundles administered %s", installedBundles.keySet());
-
-		// From now on, the bundles are on their own. They have
-		// by default AllPermission, but if they install bundles
-		// they will not automatically get AllPermission anymore
-		if (security)
-			policy.setDefaultPermissions(null);
-
-		// Get the resolved status
-		if (padmin != null && padmin.resolveBundles(null) == false) {
-			error("could not resolve the bundles");
-			// return LauncherConstants.RESOLVE_ERROR;
-		}
-
-		// Now start all the installed bundles in the same order
-		// (unless they're a fragment)
-		
-		
-		trace("Will start bundles: %s" ,tobestarted);
-		for (Bundle b : tobestarted) {
+	/**
+	 * Install the bundles from the current jar.
+	 * 
+	 * @param tobestarted
+	 * @throws BundleException
+	 * @throws IOException
+	 */
+	void installEmbedded(List<Bundle> tobestarted) throws BundleException, IOException {
+		trace("starting in embedded mode");
+		BundleContext context = systemBundle.getBundleContext();
+		for (Object o : parms.runbundles) {
+			String path = (String) o;
+			trace("installing %s", path);
+			InputStream in = getClass().getClassLoader().getResourceAsStream(path);
 			try {
-				trace("starting %s", b.getSymbolicName());
-				if (!isFragment(b))
-					b.start(Bundle.START_ACTIVATION_POLICY);
-				trace("started  %s", b.getSymbolicName());
-			} catch (BundleException e) {
-				error("Failed to start bundle %s-%s, exception %s", b.getSymbolicName(), b
-						.getVersion(), e);
+				Bundle bundle = context.installBundle(path, in);
+				tobestarted.add(bundle);
+			}
+			finally {
+				in.close();
 			}
 		}
-
 	}
 
 	Bundle install(File f) throws Exception {
@@ -317,13 +420,15 @@ public class Launcher implements ServiceListener {
 				b.update();
 			}
 			return b;
-		} catch (BundleException e) {
+		}
+		catch (BundleException e) {
 			trace("failed reference, will try to install %s with input stream", f.getAbsolutePath());
 			String reference = f.toURI().toURL().toExternalForm();
 			InputStream in = new FileInputStream(f);
 			try {
 				return context.installBundle(reference, in);
-			} finally {
+			}
+			finally {
 				in.close();
 			}
 		}
@@ -333,35 +438,38 @@ public class Launcher implements ServiceListener {
 		// Ensure we properly close in a separate thread so that
 		// we can leverage the main thread, which is required for macs
 		Thread wait = new Thread("FrameworkWaiter") {
+			@Override
 			public void run() {
 				try {
 					FrameworkEvent result = systemBundle.waitForStop(parms.timeout);
-					trace( "framework event " + result + " " + result.getType());
+					trace("framework event " + result + " " + result.getType());
 					Thread.sleep(1000);
 					switch (result.getType()) {
-					case FrameworkEvent.STOPPED:
-						System.exit(LauncherConstants.OK);
-						break;
-					case FrameworkEvent.WAIT_TIMEDOUT:
-						System.exit(LauncherConstants.TIMEDOUT);
-						break;
+						case FrameworkEvent.STOPPED :
+							System.exit(LauncherConstants.OK);
+							break;
+						case FrameworkEvent.WAIT_TIMEDOUT :
+							System.exit(LauncherConstants.TIMEDOUT);
+							break;
 
-					case FrameworkEvent.ERROR:
-						System.exit(ERROR);
-						break;
+						case FrameworkEvent.ERROR :
+							System.exit(ERROR);
+							break;
 
-					case FrameworkEvent.WARNING:
-						System.exit(WARNING);
-						break;
+						case FrameworkEvent.WARNING :
+							System.exit(WARNING);
+							break;
 
-					case FrameworkEvent.STOPPED_BOOTCLASSPATH_MODIFIED:
-					case FrameworkEvent.STOPPED_UPDATE:
-						System.exit(UPDATE_NEEDED);
-						break;
+						case FrameworkEvent.STOPPED_BOOTCLASSPATH_MODIFIED :
+						case FrameworkEvent.STOPPED_UPDATE :
+							System.exit(UPDATE_NEEDED);
+							break;
 					}
-				} catch (InterruptedException e) {
+				}
+				catch (InterruptedException e) {
 					System.exit(CANCELED);
-				} finally {
+				}
+				finally {
 					System.err.println("System exiting due to timeout of " + parms.timeout);
 				}
 			}
@@ -371,15 +479,17 @@ public class Launcher implements ServiceListener {
 
 	private void doSecurity() {
 		try {
-			PermissionInfo allPermissions[] = new PermissionInfo[] { new PermissionInfo(
-					AllPermission.class.getName(), null, null) };
+			PermissionInfo allPermissions[] = new PermissionInfo[] {
+				new PermissionInfo(AllPermission.class.getName(), null, null)
+			};
 			policy = new SimplePermissionPolicy(this, systemBundle.getBundleContext());
 
 			// All bundles installed from the script are getting AllPermission
 			// for now.
 			policy.setDefaultPermissions(allPermissions);
 			security = true;
-		} catch (Throwable t) {
+		}
+		catch (Throwable t) {
 			// This can throw a linkage error when the framework
 			// does not carry the PermissionAdmin class
 			security = false;
@@ -413,19 +523,27 @@ public class Launcher implements ServiceListener {
 	private Framework createFramework() throws Exception {
 		Properties p = new Properties();
 		p.putAll(properties);
-		File workingdir = new File("tmp").getAbsoluteFile();
+		File workingdir = null;
 		if (parms.storageDir != null)
 			workingdir = parms.storageDir;
+		else if (parms.keep && parms.name != null) {
+			workingdir = new File(bnd, parms.name);
+		}
 
-		trace("using working dir: %s", parms.storageDir);
+		if (workingdir == null)
+			workingdir = File.createTempFile("osgi.", ".fw");
 
-		if (!parms.keep) {
+		trace("using working dir: %s", workingdir);
+
+		if (!parms.keep && workingdir.exists()) {
 			trace("deleting working dir %s because not kept", workingdir);
 			delete(workingdir);
 			p.setProperty(Constants.FRAMEWORK_STORAGE_CLEAN, "true");
 		}
 
-		workingdir.mkdirs();
+		if (!workingdir.exists() && !workingdir.mkdirs()) {
+			throw new IOException("Could not create directory " + workingdir);
+		}
 		if (!workingdir.isDirectory())
 			throw new IllegalArgumentException("Cannot create a working dir: " + workingdir);
 
@@ -435,6 +553,11 @@ public class Launcher implements ServiceListener {
 			p.setProperty(Constants.FRAMEWORK_SYSTEMPACKAGES_EXTRA, parms.systemPackages);
 			trace("system packages used: %s", parms.systemPackages);
 		}
+		
+		if (parms.systemCapabilities != null) {
+			p.setProperty(FRAMEWORK_SYSTEM_CAPABILITIES_EXTRA, parms.systemCapabilities);
+			trace("system capabilities used: %s", parms.systemCapabilities);
+		}
 
 		Framework systemBundle;
 
@@ -445,17 +568,16 @@ public class Launcher implements ServiceListener {
 			ClassLoader loader = getClass().getClassLoader();
 
 			// 3) Lookup in META-INF/services
-			List<String> implementations = getMetaInfServices(loader, FrameworkFactory.class
-					.getName());
+			List<String> implementations = getMetaInfServices(loader, FrameworkFactory.class.getName());
 
 			if (implementations.size() == 0)
 				error("Found no fw implementation");
 			if (implementations.size() > 1)
 				error("Found more than one framework implementations: %s", implementations);
 
-			String implementation = (String) implementations.get(0);
+			String implementation = implementations.get(0);
 
-			Class<?> clazz = loader.loadClass(implementation);
+			Class< ? > clazz = loader.loadClass(implementation);
 			FrameworkFactory factory = (FrameworkFactory) clazz.newInstance();
 			trace("Framework factory %s", factory);
 			systemBundle = factory.newFramework(p);
@@ -466,6 +588,27 @@ public class Launcher implements ServiceListener {
 			systemBundle = new MiniFramework(p);
 		}
 		systemBundle.init();
+
+		try {
+			systemBundle.getBundleContext().addFrameworkListener(new FrameworkListener() {
+
+				public void frameworkEvent(FrameworkEvent event) {
+					switch (event.getType()) {
+						case FrameworkEvent.ERROR :
+						case FrameworkEvent.WAIT_TIMEDOUT :
+							trace("Refresh will end due to error or timeout %s", event.toString());
+							
+						case FrameworkEvent.PACKAGES_REFRESHED :
+							inrefresh = false;
+							trace("refresh ended");
+							break;
+					}
+				}
+			});
+		}
+		catch (Exception e) {
+			trace("could not register a framework listener: %s", e);
+		}
 		trace("inited system bundle %s", systemBundle);
 		return systemBundle;
 	}
@@ -486,13 +629,15 @@ public class Launcher implements ServiceListener {
 		List<String> factories = new ArrayList<String>();
 
 		while (e.hasMoreElements()) {
-			URL url = (URL) e.nextElement();
+			URL url = e.nextElement();
 			trace("found META-INF/services in %s", url);
 
-			InputStream in = url.openStream();
+			InputStream in = null;
+			BufferedReader rdr = null;
+			String line;
 			try {
-				BufferedReader rdr = new BufferedReader(new InputStreamReader(in, "UTF-8"));
-				String line;
+				in = url.openStream();
+				rdr = new BufferedReader(new InputStreamReader(in, "UTF-8"));
 				while ((line = rdr.readLine()) != null) {
 					trace(line);
 					line = line.trim();
@@ -500,8 +645,14 @@ public class Launcher implements ServiceListener {
 						factories.add(line);
 					}
 				}
-			} finally {
-				in.close();
+			}
+			finally {
+				if (rdr != null) {
+					rdr.close();
+				}
+				if (in != null) {
+					in.close();
+				}
 			}
 		}
 		return factories;
@@ -515,8 +666,7 @@ public class Launcher implements ServiceListener {
 		String path = f.getAbsolutePath();
 		char first = path.charAt(0);
 		if (path.equals("/") || (first >= 'A' && first <= 'Z' && path.substring(1).equals(":\\")))
-			throw new IllegalArgumentException(
-					"You can not make the root the storage area because it will be deleted");
+			throw new IllegalArgumentException("You can not make the root the storage area because it will be deleted");
 		if (f.isDirectory()) {
 			File fs[] = f.listFiles();
 			for (int i = 0; i < fs.length; i++)
@@ -535,11 +685,11 @@ public class Launcher implements ServiceListener {
 			row(out, "Keep", parms.keep);
 			row(out, "Security", security);
 			list(out, fill("Run bundles", 40), parms.runbundles);
-			list(out, fill("Classpath", 40), split(System.getProperty("java.class.path"),
-					File.pathSeparator));
+			list(out, fill("Classpath", 40), split(System.getProperty("java.class.path"), File.pathSeparator));
 			list(out, fill("System Packages", 40), split(parms.systemPackages, ","));
+			list(out, fill("System Capabilities", 40), split(parms.systemCapabilities, ","));
 			row(out, "Properties");
-			for (Entry<Object, Object> entry : properties.entrySet()) {
+			for (Entry<Object,Object> entry : properties.entrySet()) {
 				String key = (String) entry.getKey();
 				String value = (String) entry.getValue();
 				row(out, key, value);
@@ -561,7 +711,7 @@ public class Launcher implements ServiceListener {
 							out.print(fill(toDate(f.lastModified()), 14));
 						else
 							out.print(fill("<>", 14));
-						
+
 						if (errors.containsKey(bundles[i])) {
 							out.print(fill(loc, 50));
 							out.print(errors.get(bundles[i]).getMessage());
@@ -572,7 +722,8 @@ public class Launcher implements ServiceListener {
 					}
 				}
 			}
-		} catch (Throwable t) {
+		}
+		catch (Throwable t) {
 			error("Sorry, can't print framework: %s", t);
 		}
 	}
@@ -581,9 +732,9 @@ public class Launcher implements ServiceListener {
 		boolean fill = true;
 		for (Object p : parms) {
 			if (fill)
-				out.print(fill(p.toString(), 40));
+				out.print(fill(p + "", 40));
 			else
-				out.print(p.toString());
+				out.print(p);
 			fill = false;
 		}
 		out.println();
@@ -592,9 +743,8 @@ public class Launcher implements ServiceListener {
 	String toDate(long t) {
 		Calendar c = Calendar.getInstance();
 		c.setTimeInMillis(t);
-		return fill(c.get(Calendar.YEAR), 4) + fill(c.get(Calendar.MONTH), 2)
-				+ fill(c.get(Calendar.DAY_OF_MONTH), 2) + fill(c.get(Calendar.HOUR_OF_DAY), 2)
-				+ fill(c.get(Calendar.MINUTE), 2);
+		return fill(c.get(Calendar.YEAR), 4) + fill(c.get(Calendar.MONTH), 2) + fill(c.get(Calendar.DAY_OF_MONTH), 2)
+				+ fill(c.get(Calendar.HOUR_OF_DAY), 2) + fill(c.get(Calendar.MINUTE), 2);
 	}
 
 	private String fill(int n, int width) {
@@ -606,7 +756,7 @@ public class Launcher implements ServiceListener {
 	}
 
 	private String fill(String s, int width, char filler, int dir) {
-		StringBuffer sb = new StringBuffer();
+		StringBuilder sb = new StringBuilder();
 		if (s.length() > width) {
 			int half = (width - 1) / 2;
 			return s.substring(0, half) + ".." + s.substring(s.length() - half);
@@ -628,23 +778,23 @@ public class Launcher implements ServiceListener {
 
 	private String toState(int state) {
 		switch (state) {
-		case Bundle.INSTALLED:
-			return "INSTL";
-		case Bundle.RESOLVED:
-			return "RSLVD";
-		case Bundle.STARTING:
-			return "STRTD";
-		case Bundle.STOPPING:
-			return "STPPD";
-		case Bundle.ACTIVE:
-			return "ACTIV";
-		case Bundle.UNINSTALLED:
-			return "UNNST";
+			case Bundle.INSTALLED :
+				return "INSTL";
+			case Bundle.RESOLVED :
+				return "RSLVD";
+			case Bundle.STARTING :
+				return "STRTD";
+			case Bundle.STOPPING :
+				return "STPPD";
+			case Bundle.ACTIVE :
+				return "ACTIV";
+			case Bundle.UNINSTALLED :
+				return "UNNST";
 		}
 		return "? " + state;
 	}
 
-	private void list(PrintStream out, String del, List<?> l) {
+	private void list(PrintStream out, String del, List< ? > l) {
 		for (Object o : l) {
 			String s = o.toString();
 			out.print(del);
@@ -655,47 +805,47 @@ public class Launcher implements ServiceListener {
 
 	public int translateToError(BundleException e) {
 		switch (e.getType()) {
-		case BundleException.ACTIVATOR_ERROR:
-			return LauncherConstants.ACTIVATOR_ERROR;
+			case BundleException.ACTIVATOR_ERROR :
+				return LauncherConstants.ACTIVATOR_ERROR;
 
-		case BundleException.DUPLICATE_BUNDLE_ERROR:
-			return LauncherConstants.DUPLICATE_BUNDLE;
+			case BundleException.DUPLICATE_BUNDLE_ERROR :
+				return LauncherConstants.DUPLICATE_BUNDLE;
 
-		case BundleException.RESOLVE_ERROR:
-			return LauncherConstants.RESOLVE_ERROR;
+			case BundleException.RESOLVE_ERROR :
+				return LauncherConstants.RESOLVE_ERROR;
 
-		case BundleException.INVALID_OPERATION:
-		case BundleException.MANIFEST_ERROR:
-		case BundleException.NATIVECODE_ERROR:
-		case BundleException.STATECHANGE_ERROR:
-		case BundleException.UNSUPPORTED_OPERATION:
-		case BundleException.UNSPECIFIED:
-		default:
-			return ERROR;
+			case BundleException.INVALID_OPERATION :
+			case BundleException.MANIFEST_ERROR :
+			case BundleException.NATIVECODE_ERROR :
+			case BundleException.STATECHANGE_ERROR :
+			case BundleException.UNSUPPORTED_OPERATION :
+			case BundleException.UNSPECIFIED :
+			default :
+				return ERROR;
 		}
 	}
 
 	public String translateToMessage(BundleException e) {
 		switch (e.getType()) {
-		case BundleException.ACTIVATOR_ERROR:
-			Throwable t = e.getCause();
-			StackTraceElement[] stackTrace = t.getStackTrace();
-			if (stackTrace == null || stackTrace.length == 0)
-				return "activator error " + t.getMessage();
-			StackTraceElement top = stackTrace[0];
-			return "activator error " + t.getMessage() + " from: " + top.getClassName() + ":"
-					+ top.getMethodName() + "#" + top.getLineNumber();
+			case BundleException.ACTIVATOR_ERROR :
+				Throwable t = e.getCause();
+				StackTraceElement[] stackTrace = t.getStackTrace();
+				if (stackTrace == null || stackTrace.length == 0)
+					return "activator error " + t.getMessage();
+				StackTraceElement top = stackTrace[0];
+				return "activator error " + t.getMessage() + " from: " + top.getClassName() + ":" + top.getMethodName()
+						+ "#" + top.getLineNumber();
 
-		case BundleException.DUPLICATE_BUNDLE_ERROR:
-		case BundleException.RESOLVE_ERROR:
-		case BundleException.INVALID_OPERATION:
-		case BundleException.MANIFEST_ERROR:
-		case BundleException.NATIVECODE_ERROR:
-		case BundleException.STATECHANGE_ERROR:
-		case BundleException.UNSUPPORTED_OPERATION:
-		case BundleException.UNSPECIFIED:
-		default:
-			return e.getMessage();
+			case BundleException.DUPLICATE_BUNDLE_ERROR :
+			case BundleException.RESOLVE_ERROR :
+			case BundleException.INVALID_OPERATION :
+			case BundleException.MANIFEST_ERROR :
+			case BundleException.NATIVECODE_ERROR :
+			case BundleException.STATECHANGE_ERROR :
+			case BundleException.UNSUPPORTED_OPERATION :
+			case BundleException.UNSPECIFIED :
+			default :
+				return e.getMessage();
 		}
 	}
 
@@ -703,11 +853,13 @@ public class Launcher implements ServiceListener {
 
 	class AllPolicy extends Policy {
 
+		@Override
 		public PermissionCollection getPermissions(CodeSource codesource) {
 			trace("Granting AllPermission to %s", codesource.getLocation());
 			return all;
 		}
 
+		@Override
 		public void refresh() {
 			trace("Policy refresh");
 		}
@@ -725,13 +877,15 @@ public class Launcher implements ServiceListener {
 			setReadOnly();
 		}
 
-		public void add(Permission permission) {
-		}
+		@Override
+		public void add(Permission permission) {}
 
+		@Override
 		public Enumeration<Permission> elements() {
 			return list.elements();
 		}
 
+		@Override
 		public boolean implies(Permission permission) {
 			return true;
 		}
@@ -739,8 +893,7 @@ public class Launcher implements ServiceListener {
 
 	public synchronized void serviceChanged(ServiceEvent event) {
 		if (event.getType() == ServiceEvent.REGISTERED) {
-			mainThread = (Runnable) systemBundle.getBundleContext().getService(
-					event.getServiceReference());
+			mainThread = (Runnable) systemBundle.getBundleContext().getService(event.getServiceReference());
 			notifyAll();
 		}
 	}
@@ -754,7 +907,7 @@ public class Launcher implements ServiceListener {
 	private void message(String prefix, String string, Object[] objects) {
 		Throwable e = null;
 
-		StringBuffer sb = new StringBuffer();
+		StringBuilder sb = new StringBuilder();
 		int n = 0;
 		sb.append(prefix);
 		for (int i = 0; i < string.length(); i++) {
@@ -762,28 +915,28 @@ public class Launcher implements ServiceListener {
 			if (c == '%') {
 				c = string.charAt(++i);
 				switch (c) {
-				case 's':
-					if (n < objects.length) {
-						Object o = objects[n++];
-						if (o instanceof Throwable) {
-							e = (Throwable) o;
-							if (o instanceof BundleException) {
-								sb.append(translateToMessage((BundleException) o));
-							} else if (o instanceof InvocationTargetException) {
-								Throwable t = (InvocationTargetException) o;
-								sb.append(t.getMessage());
-								e = t;
-							} else
-								sb.append(e.getMessage());
-						} else {
-							sb.append(o);
-						}
-					} else
-						sb.append("<no more arguments>");
-					break;
+					case 's' :
+						if (n < objects.length) {
+							Object o = objects[n++];
+							if (o instanceof Throwable) {
+								e = (Throwable) o;
+								if (o instanceof BundleException) {
+									sb.append(translateToMessage((BundleException) o));
+								} else if (o instanceof InvocationTargetException) {
+									Throwable t = (InvocationTargetException) o;
+									sb.append(t.getMessage());
+									e = t;
+								} else
+									sb.append(e.getMessage());
+							} else {
+								sb.append(o);
+							}
+						} else
+							sb.append("<no more arguments>");
+						break;
 
-				default:
-					sb.append(c);
+					default :
+						sb.append(c);
 				}
 			} else {
 				sb.append(c);
